'use server';

import fs from 'fs/promises'; // 使用异步版本的 fs 模块
import path from 'path';
import axios from 'axios';

let giteaRepoUrl: string | undefined;
let giteaToken: string | undefined;
let testDataPath: string = 'testdata'; // 默认测试数据目录

/**
 * 设置 Gitea 仓库 URL
 * @param url 仓库 URL
 */
export const setGiteaRepoUrl = async (url: string) => {
    if (!url || !url.startsWith('http')) {
        throw new Error('Invalid repository URL. Please provide a valid URL starting with "http".');
    }
    giteaRepoUrl = url;
};

/**
 * 设置 Gitea Token
 * @param token 仓库 API 访问令牌
 */
export const setGiteaToken = async (token: string) => {
    if (!token || token.trim().length === 0) {
        throw new Error('Invalid token. Please provide a non-empty token.');
    }
    giteaToken = token;
};

/**
 * 设置测试数据在仓库中的路径
 * @param path 测试数据目录路径
 */
export const setTestDataPath = async (path: string) => {
    if (!path || path.trim().length === 0) {
        throw new Error('Invalid test data path. Please provide a non-empty path.');
    }
    testDataPath = path;
};

/**
 * 从 Gitea 仓库中下载指定目录的文件
 * @param dir 目录路径
 * @param tempDir 本地存储的临时目录
 */
export const fetchFilesFromGitea = async (dir: string, tempDir: string) => {
    if (!giteaRepoUrl || !giteaToken) {
        throw new Error('Gitea repository URL or token is not set. Please configure them using setGiteaRepoUrl and setGiteaToken.');
    }

    const targetDir = path.join(tempDir, dir);
    await fs.mkdir(targetDir, { recursive: true });

    try {
        const response = await axios.get(`${giteaRepoUrl}/contents/${dir}`, {
            headers: {
                Authorization: `token ${giteaToken}`,
            },
        });

        const files = response.data;
        if (!Array.isArray(files)) {
            throw new Error(`Invalid directory structure for ${dir} in repository.`);
        }

        // 并行下载文件
        await Promise.all(files.filter(file => file.type === 'file').map(async (file) => {
            const fileResponse = await axios.get(file.download_url, {
                headers: {
                    Authorization: `token ${giteaToken}`,
                },
                responseType: 'arraybuffer',
            });

            const filePath = path.join(targetDir, file.name);
            await fs.writeFile(filePath, fileResponse.data);
        }));
    } catch (error) {
        throw new Error(`Failed to fetch files from '${dir}': ${(error as Error).message}`);
    }
};

/**
 * 比较运行结果与正确答案文件
 * @param resultFilePath 运行结果文件路径
 * @param expectedFilePath 正确答案文件路径
 * @returns 是否一致的比较结果
 */
export const compareResults = async (resultFilePath: string, expectedFilePath: string): Promise<boolean> => {
    try {
        const result = (await fs.readFile(resultFilePath, 'utf-8')).trim();
        const expected = (await fs.readFile(expectedFilePath, 'utf-8')).trim();
        return result === expected;
    } catch (error) {
        throw new Error(`Failed to read files for comparison: ${(error as Error).message}`);
    }
};
